;**************************************************************************
;*          1MHz LED Frequency Counter              *
; 
; The 7 segments are at RB0 to RB5,RA3. 
; Each digit is stored in its reg. and converted to 7 segments by preset
; values in RAMs 20-3A.
; TMR2 is set to 4 mS as time base for the clock.
; Conversion to decimal by counting TMR1L reg once and then 
; counting TMR1H reg 256 times.
;
; This code is free to use, for more info   http://www.moty22.co.uk      
;**************************************************************************
#include <p16F628.inc>


;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_CP_ALL                      EQU     H'03FF'
_CP_75                       EQU     H'17FF'
_CP_50                       EQU     H'2BFF'
_CP_OFF                      EQU     H'3FFF'
_DATA_CP_ON                  EQU     H'3EFF'
_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'

	__CONFIG      _BODEN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _XT_OSC  ; 

;==========================================================================
;       Variable Definition
;==========================================================================
d0  	EQU	H'20'		
d1		EQU	H'21'		
d2      EQU H'22'
d3      EQU H'23'
d4      EQU H'24'
d5      EQU H'25'
d6		EQU	H'26'		
d7		EQU	H'27'		
d8      EQU H'28'
d9      EQU H'29'
dF      EQU H'2A'
d10  	EQU	H'30'		
d11		EQU	H'31'		
d12     EQU H'32'
d13     EQU H'33'
d14     EQU H'34'
d15     EQU H'35'
d16		EQU	H'36'		
d17		EQU	H'37'		
d18     EQU H'38'
d19     EQU H'39'
dFF     EQU H'3A'
dgt100	EQU	H'3B'		
dgt10   EQU H'2B'
dgt1    EQU H'2F'
TIMER1	EQU	H'2C'		
TIMER2  EQU H'2D'
count   EQU H'2E'
cnvrt   EQU H'3C'


		ORG	0		;Reset vector address
		GOTO	init		

DELAY1  movlw D'125'     ;5mS 
    	movwf timer1
    	movlw D'7'
    	movwf timer2
LOOP1   decfsz timer1, f
	    goto loop1
	    decfsz timer2, f
	    goto loop1
		return
		
dsp1  	movfw   dgt100    ;7 segments converter
    	addlw   H'20'     ;lookup table 
    	movwf   FSR       
    	movfw   INDF
    	movwf   PORTB     ;portb outputs the selected segments
    	movfw   dgt100    ;7 segments converter
    	addlw   H'30'     ;lookup table 
    	movwf   FSR       
    	movfw   INDF
    	movwf   PORTA     ;porta outputs segment G
    	return
    	
dsp2  	movfw   dgt10    ;7 segments converter
    	addlw   H'20'     ;lookup table 
    	movwf   FSR       
    	movfw   INDF
    	movwf   PORTB     ;portb outputs the selected segments
    	movfw   dgt10    ;7 segments converter
    	addlw   H'30'     ;lookup table 
    	movwf   FSR       
    	movfw   INDF
    	movwf   PORTA     ;porta outputs segment G
    	return
    	
dsp3  	movfw   dgt1    ;7 segments converter
    	addlw   H'20'     ;lookup table 
    	movwf   FSR       
    	movfw   INDF
    	movwf   PORTB     ;portb outputs the selected segments
    	movfw   dgt1    ;7 segments converter
    	addlw   H'30'     ;lookup table 
    	movwf   FSR       
    	movfw   INDF
    	movwf   PORTA     ;porta outputs segment G
    	return    	
    	
init	MOVLW	B'00000111'	;Disable Comparator module's
		MOVWF	CMCON
		movlw   B'11101'      ;11101
		movwf   T2CON        ;TMR2 prescale=4, postscale=1/4
		movlw   B'101110'    ;101110
		movwf   T1CON        ;TMR1 prescale=4, osc=on, counter mode  
		BSF	    STATUS,RP0	;Switch to bank 1
		movlw   .250
		movwf   PR2         ;TMR2 period 4ms 
		MOVLW	B'100'	    ;TMR0 prescale-1:32
		MOVWF	OPTION_REG
		movlw   B'11000000'	
		movwf	TRISB		 ;RB 
		MOVLW	B'11100000'	 ;RA 0-4 outputs
		MOVWF	TRISA
		BCF	    STATUS,RP0	;Switch to Bank 0

        movlw   B'111111'   ;load reg with 7 segments data
        movwf   d0          ;segments for 0
        clrf    d10
        movlw   B'110'
        movwf   d1          ;segments for 1
        clrf    d11
        movlw   B'11011'
        movwf   d2
        clrf    d12
        bsf     d12,3
        movlw   B'1111'
        movwf   d3
        clrf    d13
        bsf     d13,3
        movlw   B'100110'
        movwf   d4
        clrf    d14
        bsf     d14,3
        movlw   B'101101'
        movwf   d5
        clrf    d15
        bsf     d15,3
        movlw   B'111100'
        movwf   d6
        clrf    d16
        bsf     d16,3
        movlw   B'111'
        movwf   d7
        clrf    d17
        movlw   B'111111'
        movwf   d8
        clrf    d18
        bsf     d18,3
        movlw   B'100111'
        movwf   d9         ;segments for 9
        clrf    d19
        bsf     d19,3
        movlw   B'110001'
        movwf   dF         ;segments for F
        clrf    dFF
        bsf     dFF,3
        
        clrf    dgt1
        clrf    dgt10
        clrf    dgt100

main    movlw   .120
        movwf   count
loop2   call    dsp1         ;mux digits
        bsf     PORTA,0      ;digit1 drive
        call    delay1      ;sets the mux to 100Hz 
        call    dsp2
        bsf     PORTA,1
        call    delay1
        call    dsp3         ;mux digits
        bsf     PORTA,2      ;digit3 drive
        call    delay1       ;sets the mux to 100Hz 
        decfsz  count,f
        goto    loop2        ;loop for 1 sec

        bcf     PIR1,TMR2IF  ;clear gate flag
        clrf    TMR1L
        clrf    TMR1H
gopen   btfss   PIR1,TMR2IF     ;gate open
        goto    gopen
        bsf     T1CON,TMR1ON    ;start count
        bcf     PIR1,TMR2IF
gclse   btfss   PIR1,TMR2IF     ;gate close
        goto    gclse       
        bcf     T1CON,TMR1ON    ;stop count
        bcf     PIR1,TMR2IF
        incf    TMR1H,f      ;start convert binary to decimal
        movfw   TMR1L
        movwf   cnvrt
        incf    cnvrt,f 
        clrf    dgt1
        clrf    dgt10
        clrf    dgt100

b2d     decfsz  cnvrt,f     ;count cnvrt reg
        goto    b2d1
        goto    b2d2
        
b2d1    incf    dgt1,f      ;add 1 to decimal count
        movlw   .10
    	subwf   dgt1,w
    	btfss   status,z   ;is digit1=10
    	goto    b2d
    	incf    dgt10,f     ;
    	clrf    dgt1
    	movlw   .10
    	subwf   dgt10,w
    	btfss   status,z   ;is digit10=10
    	goto    b2d
    	incf    dgt100,f
    	clrf    dgt10
        movlw   .10
    	subwf   dgt100,w
    	btfss   status,z   ;is digit100=10
    	goto    b2d
    	clrf    dgt100     ;overflow: display O
    	movlw   .10
    	movwf   dgt10       ;display F
        movlw   .120
        movwf   count
loop3   call    dsp1         ;display OF for a sec
        bsf     PORTA,0
        call    delay1
        call    dsp2
        bsf     PORTA,1
        call    delay1
        call    dsp3
        bcf     PORTA,2
        call    delay1
        decfsz  count,f
        goto    loop3    	
    	goto    main 
        
b2d2    decfsz  TMR1H,f     ;multiply by 256
        goto    b2d
        goto    main 

	    	
		END
